@@ -17,7 +17,7 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-import numpy
+from __future__ import print_function
 
 # nuitka-skip-unless-imports: numpy
 
@@ -26,9 +26,16 @@ import numpy
 
 # Make sure, the usual bad ones are not included with anti-bloat.
 
-# nuitka-project: --enable-plugin=anti-bloat
-# nuitka-project: --noinclude-setuptools-mode=error
-# nuitka-project: --noinclude-pytest-mode=error
+# nuitka-project: --noinclude-default-mode=error
 # nuitka-project: --noinclude-custom-mode=numpy.distutils:error
 
+# nuitka-project: --noinclude-custom-mode=pydoc:error
+
+# To trigger DLL usage on non-Linux.
+import numpy.core.multiarray
+
+a = numpy.arange(15).reshape(3, 5)
+
+print("An array", a)
+
 print("OK.")
\ No newline at end of file
